@@ -89,7 +89,7 @@ end
 function _instance:get(name)
     local value = self._INFO:get(name)
     if name == "configs" then
-        -- we need merge it, because current builtin configs always exists
+        -- we need to merge it, because current builtin configs always exists
         if self:base() then
             local configs_base = self:base():get("configs")
             if configs_base then
@@ -212,7 +212,7 @@ end
 -- the current platform is belong to the given platforms?
 function _instance:is_plat(...)
     local plat = self:plat()
-    for _, v in ipairs(table.join(...)) do
+    for _, v in ipairs(table.pack(...)) do
         if v and plat == v then
             return true
         end
@@ -222,7 +222,7 @@ end
 -- the current architecture is belong to the given architectures?
 function _instance:is_arch(...)
     local arch = self:arch()
-    for _, v in ipairs(table.join(...)) do
+    for _, v in ipairs(table.pack(...)) do
         if v and arch:find("^" .. v:gsub("%-", "%%-") .. "$") then
             return true
         end
@@ -247,7 +247,7 @@ end
 -- the current architecture is belong to the given target architectures?
 function _instance:is_targetarch(...)
     local targetarch = self:targetarch()
-    for _, v in ipairs(table.join(...)) do
+    for _, v in ipairs(table.pack(...)) do
         if v and targetarch:find("^" .. v:gsub("%-", "%%-") .. "$") then
             return true
         end
@@ -270,7 +270,14 @@ end
 
 -- get urls
 function _instance:urls()
-    return self._URLS or table.wrap(self:get("urls"))
+    local urls = self._URLS
+    if urls == nil then
+        urls = table.wrap(self:get("urls"))
+        if #urls == 1 and urls[1] == "" then
+            urls = {}
+        end
+    end
+    return urls
 end
 
 -- get urls
@@ -411,8 +418,12 @@ function _instance:plaindeps()
 end
 
 -- get library deps with correct link order
-function _instance:librarydeps()
-    return self._LIBRARYDEPS
+function _instance:librarydeps(opt)
+    if opt and opt.private then
+        return self._LIBRARYDEPS_WITH_PRIVATE
+    else
+        return self._LIBRARYDEPS
+    end
 end
 
 -- get parents
@@ -466,7 +477,7 @@ function _instance:sourcehash(url_alias)
         if not sourcehash then
             sourcehash = versions[version_str]
         end
-        if sourcehash then
+        if sourcehash and #sourcehash == 40 then
             sourcehash = sourcehash:lower()
         end
         return sourcehash
@@ -555,7 +566,7 @@ function _instance:is_system()
 end
 
 -- is the third-party package? e.g. brew::pcre2/libpcre2-8, conan::OpenSSL/1.0.2n@conan/stable
--- we need install and find package by third-party package manager directly
+-- we need to install and find package by third-party package manager directly
 --
 function _instance:is_thirdparty()
     return self._is_thirdparty
@@ -594,7 +605,7 @@ end
 
 -- is debug package?
 function _instance:is_debug()
-    return self:config("debug")
+    return self:config("debug") or self:config("asan")
 end
 
 -- is the supported package?
@@ -637,6 +648,11 @@ function _instance:use_external_includes()
     if external == nil then
         external = self:policy("package.include_external_headers")
     end
+    -- disable -Isystem for external packages as it seems to break. e.g. assimp
+    -- @see https://github.com/msys2/MINGW-packages/issues/10761
+    if external == nil and self:is_plat("mingw") and os.is_subhost("msys") then
+        external = false
+    end
     if external == nil then
         external = true
     end
@@ -797,6 +813,22 @@ function _instance:scriptdir()
     return self._SCRIPTDIR
 end
 
+-- get the rules directory
+function _instance:rulesdir()
+    local rulesdir = self._RULESDIR
+    if rulesdir == nil then
+        rulesdir = path.join(self:scriptdir(), "rules")
+        if not os.isdir(rulesdir) and self:base() then
+            rulesdir = self:base():rulesdir()
+        end
+        if rulesdir == nil or not os.isdir(rulesdir) then
+            rulesdir = false
+        end
+        self._RULESDIR = rulesdir
+    end
+    return rulesdir or nil
+end
+
 -- get the references info of this package
 function _instance:references()
     local references_file = path.join(self:installdir({readonly = true}), "references.txt")
@@ -870,31 +902,38 @@ function _instance:manifest_save()
 
     -- save global variables and component variables
     local vars
+    local extras
     local components
     local apis = language.apis()
-    for _, apiname in ipairs(table.join(apis.values, apis.paths)) do
+    for _, apiname in ipairs(table.join(apis.values, apis.paths, apis.groups)) do
         if apiname:startswith("package.add_") or apiname:startswith("package.set_")  then
             local name = apiname:sub(13)
-            local value = self:get(name)
-            if value ~= nil then
+            local values = self:get(name)
+            if values ~= nil then
                 vars = vars or {}
-                vars[name] = value
+                vars[name] = values
+                local extra = self:extraconf(name)
+                if extra then
+                    extras = extras or {}
+                    extras[name] = extra
+                end
             end
             for _, component_name in ipairs(table.wrap(self:get("components"))) do
                 local comp = self:component(component_name)
                 if comp then
-                    local component_value = comp:get(name)
-                    if component_value ~= nil then
+                    local component_values = comp:get(name)
+                    if component_values ~= nil then
                         components = components or {}
                         components.vars = components.vars or {}
                         components.vars[component_name] = components.vars[component_name] or {}
-                        components.vars[component_name][name] = component_value
+                        components.vars[component_name][name] = component_values
                     end
                 end
             end
         end
     end
     manifest.vars = vars
+    manifest.extras = extras
     manifest.components = components
 
     -- save repository
@@ -919,7 +958,7 @@ function _instance:manifest_save()
     end
 
     -- save manifest
-    local ok, errors = io.save(self:manifest_file(), manifest)
+    local ok, errors = io.save(self:manifest_file(), manifest, { orderkeys = true })
     if not ok then
         os.raise(errors)
     end
@@ -1066,7 +1105,7 @@ function _instance:build_envs(lazy_loading)
         for _, opt in ipairs(table.join(language_menu.options("config"), platform_menu.options("config"))) do
             local optname = opt[2]
             if type(optname) == "string" then
-                -- we need only index it to force load it's value
+                -- we only need to index it to force load it's value
                 local value = build_envs[optname]
             end
         end
@@ -1327,9 +1366,44 @@ function _instance:configs()
             configs = {}
             local requireinfo = self:requireinfo()
             local configs_required = requireinfo and requireinfo.configs or {}
-            local ignored_configs = hashset.from(requireinfo and requireinfo.ignored_configs or {})
             for _, name in ipairs(table.wrap(configs_defined)) do
-                if not ignored_configs:has(name) then
+                local value = configs_required[name]
+                if value == nil then
+                    value = self:extraconf("configs", name, "default")
+                end
+                configs[name] = value
+            end
+        else
+            configs = false
+        end
+        self._CONFIGS = configs
+    end
+    return configs and configs or nil
+end
+
+-- get the given configuration value of package for buildhash
+function _instance:_config_for_buildhash(name)
+    local value
+    local configs = self:_configs_for_buildhash()
+    if configs then
+        value = configs[name]
+    end
+    return value
+end
+
+-- get the configurations of package for buildhash
+-- @note on_test still need these configs
+function _instance:_configs_for_buildhash()
+    local configs = self._CONFIGS_FOR_BUILDHASH
+    if configs == nil then
+        local configs_defined = self:get("configs")
+        if configs_defined then
+            configs = {}
+            local requireinfo = self:requireinfo()
+            local configs_required = requireinfo and requireinfo.configs or {}
+            local ignored_configs_for_buildhash = hashset.from(requireinfo and requireinfo.ignored_configs_for_buildhash or {})
+            for _, name in ipairs(table.wrap(configs_defined)) do
+                if not ignored_configs_for_buildhash:has(name) then
                     local value = configs_required[name]
                     if value == nil then
                         value = self:extraconf("configs", name, "default")
@@ -1340,7 +1414,7 @@ function _instance:configs()
         else
             configs = false
         end
-        self._CONFIGS = configs
+        self._CONFIGS_FOR_BUILDHASH = configs
     end
     return configs and configs or nil
 end
@@ -1388,7 +1462,7 @@ function _instance:buildhash()
                     str = str .. "_" .. table.concat(hashs, "_")
                 end
             end
-            local toolchains = self:config("toolchains")
+            local toolchains = self:_config_for_buildhash("toolchains")
             if opt.toolchains ~= false and toolchains then
                 toolchains = table.copy(table.wrap(toolchains))
                 table.sort(toolchains)
@@ -1407,8 +1481,8 @@ function _instance:buildhash()
 
         -- we need to be compatible with the hash value string for the previous xmake version
         -- without builtin pic configuration (< 2.5.1).
-        if self:config("pic") then
-            local configs = table.copy(self:configs())
+        if self:_config_for_buildhash("pic") then
+            local configs = table.copy(self:_configs_for_buildhash())
             configs.pic = nil
             buildhash = _get_buildhash(configs, {sourcehash = false, toolchains = false})
             if not os.isdir(_get_installdir(buildhash)) then
@@ -1419,7 +1493,7 @@ function _instance:buildhash()
         -- we need to be compatible with the hash value string for the previous xmake version
         -- without sourcehash (< 2.5.2)
         if not buildhash then
-            buildhash = _get_buildhash(self:configs(), {sourcehash = false, toolchains = false})
+            buildhash = _get_buildhash(self:_configs_for_buildhash(), {sourcehash = false, toolchains = false})
             if not os.isdir(_get_installdir(buildhash)) then
                 buildhash = nil
             end
@@ -1428,7 +1502,7 @@ function _instance:buildhash()
         -- we need to be compatible with the previous xmake version
         -- without toolchains (< 2.6.4)
         if not buildhash then
-            buildhash = _get_buildhash(self:configs(), {toolchains = false})
+            buildhash = _get_buildhash(self:_configs_for_buildhash(), {toolchains = false})
             if not os.isdir(_get_installdir(buildhash)) then
                 buildhash = nil
             end
@@ -1436,7 +1510,7 @@ function _instance:buildhash()
 
         -- get build hash for current version
         if not buildhash then
-            buildhash = _get_buildhash(self:configs())
+            buildhash = _get_buildhash(self:_configs_for_buildhash())
         end
         self._BUILDHASH = buildhash
     end
@@ -1544,7 +1618,8 @@ function _instance:_fetch_tool(opt)
         else
             fetchinfo = self:find_tool(self:name(), {require_version = opt.require_version,
                                                      cachekey = "fetch_package_xmake",
-                                                     norun = true, -- we need not run it to check for xmake/packages, @see https://github.com/xmake-io/xmake-repo/issues/66
+                                                     norun = true, -- we don't need to run it to check for xmake/packages, @see https://github.com/xmake-io/xmake-repo/issues/66
+                                                     system = false, -- we only find it from xmake/packages, @see https://github.com/xmake-io/xmake-repo/pull/2085
                                                      force = opt.force})
 
             -- may be toolset, not single tool
@@ -1641,6 +1716,7 @@ function _instance:find_tool(name, opt)
                                   version = true, -- we alway check version
                                   require_version = opt.require_version,
                                   norun = opt.norun,
+                                  system = opt.system,
                                   force = opt.force})
 end
 
@@ -1666,6 +1742,7 @@ function _instance:find_package(name, opt)
                               buildhash = self:buildhash(), -- for xmake package or 3rd package manager, e.g. go:: ..
                               cachekey = opt.cachekey or "fetch_package_system",
                               external = opt.external,
+                              includes = opt.includes, -- system/find_package need it
                               system = system})
 end
 
@@ -1710,9 +1787,6 @@ function _instance:fetch(opt)
         -- we need ignore `{system = true/false}` argument if be 3rd package
         -- @see https://github.com/xmake-io/xmake/issues/726
         system = nil
-    elseif self:is_cross() then
-        -- we need disable system package for cross-compilation
-        system = false
     end
 
     -- use sysincludedirs/-isystem instead of -I?
@@ -1736,8 +1810,8 @@ function _instance:fetch(opt)
             end
         end
 
-        -- fetch it from the system directories
-        if not fetchinfo and system ~= false then
+        -- fetch it from the system directories (disabled for cross-compilation)
+        if not fetchinfo and system ~= false and not self:is_cross() then
             fetchinfo = self:_fetch_tool({system = true, require_version = require_ver, force = opt.force})
             if fetchinfo then
                 is_system = true
@@ -1753,8 +1827,8 @@ function _instance:fetch(opt)
             end
         end
 
-        -- fetch it from the system and external package sources
-        if not fetchinfo and system ~= false then
+        -- fetch it from the system and external package sources (disabled for cross-compilation)
+        if not fetchinfo and system ~= false and not self:is_cross() then
             fetchinfo = self:_fetch_library({system = true, require_version = require_ver, external = external, force = opt.force})
             if fetchinfo then
                 is_system = true
@@ -1765,7 +1839,7 @@ function _instance:fetch(opt)
     -- save to cache
     self._FETCHINFO = fetchinfo
 
-    -- we need update the real version if it's system package
+    -- we need to update the real version if it's system package
     -- @see https://github.com/xmake-io/xmake/issues/3333
     if is_system and fetchinfo and fetchinfo.version then
         local fetch_version = semver.new(fetchinfo.version)
@@ -2045,6 +2119,24 @@ function _instance:_generate_lto_configs(sourcekind)
     return configs
 end
 
+-- generate sanitizer configs
+function _instance:_generate_sanitizer_configs(checkmode, sourcekind)
+
+    -- add cflags
+    local configs = {}
+    if sourcekind and self:has_tool(sourcekind, "cl", "clang", "clangxx", "gcc", "gxx") then
+        local cflag = sourcekind == "cxx" and "cxxflags" or "cflags"
+        configs[cflag] = "-fsanitize=" .. checkmode
+    end
+
+    -- add ldflags and shflags
+    if self:has_tool("ld", "link", "clang", "clangxx", "gcc", "gxx") then
+        configs.ldflags = "-fsanitize=" .. checkmode
+        configs.shflags = "-fsanitize=" .. checkmode
+    end
+    return configs
+end
+
 -- generate building configs for has_xxx/check_xxx
 function _instance:_generate_build_configs(configs, opt)
     opt = opt or {}
@@ -2080,10 +2172,32 @@ function _instance:_generate_build_configs(configs, opt)
             end
         end
     end
+    if self:config("asan") then
+        local configs_asan = self:_generate_sanitizer_configs("address", opt.sourcekind or "cxx")
+        if configs_asan then
+            for k, v in pairs(configs_asan) do
+                configs[k] = table.wrap(configs[k] or {})
+                table.join2(configs[k], v)
+            end
+        end
+    end
     -- enable exceptions for msvc by default
     if opt.sourcekind == "cxx" and configs.exceptions == nil and self:has_tool("cxx", "cl") then
         configs.exceptions = "cxx"
     end
+
+    -- pass user flags to on_test, because some flags need be passed to ldflags in on_test
+    -- e.g. add_requireconfs("**", {configs = {cxflags = "/fsanitize=address", ldflags = "/fsanitize=address"}})
+    --
+    -- @see https://github.com/xmake-io/xmake/issues/4046
+    --
+    for name, flags in pairs(self:configs()) do
+        if name:endswith("flags") and self:extraconf("configs", name, "builtin") then
+            configs[name] = table.wrap(configs[name] or {})
+            table.join2(configs[name], flags)
+        end
+    end
+
     if configs and (configs.ldflags or configs.shflags) then
         configs.force = {ldflags = configs.ldflags, shflags = configs.shflags}
         configs.ldflags = nil
@@ -2583,7 +2697,7 @@ function package.load_from_repository(packagename, packagedir, opt)
     -- get interpreter
     local interp = package._interpreter()
 
-    -- we need modify plat/arch in description scope at same time
+    -- we need to modify plat/arch in description scope at same time
     -- if plat/arch are passed to add_requires.
     --
     -- @see https://github.com/orgs/xmake-io/discussions/3439
