@@ -30,7 +30,7 @@ function _get_protoc(target, sourcekind)
     -- find protoc
     local protoc = target:data("protobuf.protoc")
     if not protoc and sourcekind == "cxx" then
-        protoc = find_tool("protoc")
+        protoc = find_tool("protoc", {envs = target:pkgenvs()})
         if protoc and protoc.program then
             target:data_set("protobuf.protoc", protoc.program)
         end
@@ -39,7 +39,7 @@ function _get_protoc(target, sourcekind)
     -- find protoc-c
     local protoc_c = target:data("protobuf.protoc-c")
     if not protoc_c and sourcekind == "cc" then
-        protoc_c = find_tool("protoc-c") or protoc
+        protoc_c = find_tool("protoc-c", {envs = target:pkgenvs()}) or protoc
         if protoc_c and protoc_c.program then
             target:data_set("protobuf.protoc-c", protoc_c.program)
         end
@@ -52,7 +52,7 @@ end
 -- get grpc_cpp_plugin
 function _get_grpc_cpp_plugin(target, sourcekind)
     assert(sourcekind == "cxx", "grpc_cpp_plugin only support c++")
-    local grpc_cpp_plugin = find_tool("grpc_cpp_plugin", {norun = true})
+    local grpc_cpp_plugin = find_tool("grpc_cpp_plugin", {norun = true, force = true, envs = target:pkgenvs()})
     return assert(grpc_cpp_plugin and grpc_cpp_plugin.program, "grpc_cpp_plugin not found!")
 end
 
@@ -141,9 +141,10 @@ function buildcmd(target, batchcmds, sourcefile_proto, opt, sourcekind)
         path(prefixdir and prefixdir or path.directory(sourcefile_proto), function (p) return "-I" .. p end),
         path(sourcefile_dir, function (p) return (sourcekind == "cxx" and "--cpp_out=" or "--c_out=") .. p end)
     }
-    
+
     if grpc_cpp_plugin then
-        table.insert(protoc_args, "--plugin=protoc-gen-grpc=" .. grpc_cpp_plugin_bin)
+        local extension = target:is_plat("windows") and ".exe" or ""
+        table.insert(protoc_args, "--plugin=protoc-gen-grpc=" .. grpc_cpp_plugin_bin .. extension)
         table.insert(protoc_args, path(sourcefile_dir, function (p) return ("--grpc_out=") .. p end))
     end
 
@@ -155,7 +156,7 @@ function buildcmd(target, batchcmds, sourcefile_proto, opt, sourcekind)
     if grpc_cpp_plugin then
         batchcmds:compile(sourcefile_cx_grpc, objectfile_grpc, {configs = {includedirs = sourcefile_dir}})
     end
-    
+
     -- add deps
     local depmtime = os.mtime(objectfile)
     batchcmds:add_depfiles(sourcefile_proto)
@@ -194,7 +195,7 @@ function build_batchjobs(target, batchjobs, sourcebatch, opt, sourcekind)
         moduleinfo.job = batchjobs:newjob(sourcefile, function (index, total)
             local batchcmds_ = batchcmds.new({target = target})
             buildcmd(target, batchcmds_, sourcefile, {progress = (index * 100) / total}, sourcekind)
-            batchcmds_:runcmds({dryrun = option.get("dry-run")})
+            batchcmds_:runcmds({changed = target:is_rebuilt(), dryrun = option.get("dry-run")})
         end)
     end
 
