@@ -32,7 +32,11 @@ function _link(target, linkdirs, framework, qt_sdkver)
         if target:is_plat("windows") then
             debug_suffix = "d"
         elseif target:is_plat("mingw") then
-            debug_suffix = "d"
+            if qt_sdkver:ge("5.15.2") then
+                debug_suffix = ""
+            else
+                debug_suffix = "d"
+            end
         elseif target:is_plat("android") or target:is_plat("linux") then
             debug_suffix = ""
         end
@@ -41,8 +45,8 @@ function _link(target, linkdirs, framework, qt_sdkver)
         else -- for qt4.x, e.g. QtGui4.lib
             if target:is_plat("windows", "mingw") then
                 framework = "Qt" .. framework:sub(3) .. (is_mode("debug") and debug_suffix or "") .. qt_sdkver:major()
-            else 
-                framework = "Qt" .. framework:sub(3) .. (is_mode("debug") and debug_suffix or "") 
+            else
+                framework = "Qt" .. framework:sub(3) .. (is_mode("debug") and debug_suffix or "")
             end
         end
         if target:is_plat("android") then --> -lQt5Core_armeabi/-lQt5CoreDebug_armeabi for 5.14.x
@@ -69,8 +73,12 @@ function _find_static_links_3rd(target, linkdirs, qt_sdkver, libpattern)
     for _, linkdir in ipairs(linkdirs) do
         for _, libpath in ipairs(os.files(path.join(linkdir, libpattern))) do
             local basename = path.basename(libpath)
-            -- we need ignore qt framework libraries, e.g. libQt5xxx.a, Qt5Core.lib ..
-            if not basename:startswith("libQt" .. qt_sdkver:major()) and not basename:startswith("Qt" .. qt_sdkver:major()) then
+            -- we need to ignore qt framework libraries, e.g. libQt5xxx.a, Qt5Core.lib ..
+            -- but bundled library names like libQt5Bundledxxx.a on Qt6.x
+            -- @see https://github.com/xmake-io/xmake/issues/3572
+            if basename:startswith("libQt" .. qt_sdkver:major() .. "Bundled") or (
+                (not basename:startswith("libQt" .. qt_sdkver:major())) and
+                (not basename:startswith("Qt" .. qt_sdkver:major()))) then
                 if (is_mode("debug") and basename:endswith(debug_suffix)) or (not is_mode("debug") and not basename:endswith(debug_suffix)) then
                     table.insert(links, core_target.linkname(path.filename(libpath)))
                 end
@@ -90,6 +98,8 @@ function _add_plugins(target, plugins)
         if plugin.linkdirs then
             target:values_add("qt.linkdirs", table.unpack(table.wrap(plugin.linkdirs)))
         end
+        -- TODO: add prebuilt object files in qt sdk.
+        -- these file is located at plugins/xxx/objects-Release/xxxPlugin_init/xxxPlugin_init.cpp.o
     end
 end
 
@@ -113,6 +123,22 @@ function _get_target_cppversion(target)
     end
 end
 
+-- get frameworks from target
+-- @see https://github.com/xmake-io/xmake/issues/4135
+function _get_frameworks_from_target(target)
+    local values = table.wrap(target:get("frameworks"))
+    for _, value in ipairs((target:get_from("frameworks", "option::*"))) do
+        table.join2(values, value)
+    end
+    for _, value in ipairs((target:get_from("frameworks", "package::*"))) do
+        table.join2(values, value)
+    end
+    for _, value in ipairs((target:get_from("__qt_frameworks", "dep::*", {interface = true}))) do
+        table.join2(values, value)
+    end
+    return table.unique(values)
+end
+
 -- the main entry
 function main(target, opt)
 
@@ -137,6 +163,13 @@ function main(target, opt)
         target:add("asflags", "-fPIC")
     end
 
+    if qt_sdkver:ge("6.0") then
+        -- @see https://github.com/xmake-io/xmake/issues/2071
+        if target:is_plat("windows") then
+            target:add("cxxflags", "/Zc:__cplusplus")
+            target:add("cxxflags", "/permissive-")
+        end
+    end
     -- need c++11 at least
     local languages = target:get("languages")
     local cxxlang = false
@@ -156,11 +189,6 @@ function main(target, opt)
             if (not cppversion) or (tonumber(cppversion) and tonumber(cppversion) < 17) then
                 target:add("languages", "c++17")
             end
-            -- @see https://github.com/xmake-io/xmake/issues/2071
-            if target:is_plat("windows") then
-                target:add("cxxflags", "/Zc:__cplusplus")
-                target:add("cxxflags", "/permissive-")
-            end
         else
             -- add conditionnaly c++11 to avoid for example "cl : Command line warning D9025 : overriding '/std:c++latest' with '/std:c++11'" warning
             if (not cppversion) or (tonumber(cppversion) and tonumber(cppversion) < 11) then
@@ -169,7 +197,7 @@ function main(target, opt)
         end
     end
 
-    -- add defines for the compile mode
+    -- add definitions for the compile mode
     if is_mode("debug") then
         target:add("defines", "QT_QML_DEBUG")
     elseif is_mode("release") then
@@ -202,7 +230,7 @@ function main(target, opt)
         end
     end
 
-    -- backup the user syslinks, we need add them behind the qt syslinks
+    -- backup the user syslinks, we need to add them behind the qt syslinks
     local syslinks_user = target:get("syslinks")
     target:set("syslinks", nil)
 
@@ -215,6 +243,16 @@ function main(target, opt)
     end
     target:add("syslinks", target:values("qt.links"))
 
+    -- backup qt frameworks
+    local qt_frameworks = target:get("frameworks")
+    if qt_frameworks then
+        target:set("__qt_frameworks", qt_frameworks)
+    end
+    local qt_frameworks_extra = target:extraconf("frameworks")
+    if qt_frameworks_extra then
+        target:extraconf_set("__qt_frameworks", qt_frameworks_extra)
+    end
+
     -- add frameworks
     if opt.frameworks then
         target:add("frameworks", opt.frameworks)
@@ -222,7 +260,8 @@ function main(target, opt)
 
     -- do frameworks for qt
     local frameworksset = hashset.new()
-    for _, framework in ipairs(target:get("frameworks")) do
+    local qt_frameworks = _get_frameworks_from_target(target)
+    for _, framework in ipairs(qt_frameworks) do
 
         -- translate qt frameworks
         if framework:startswith("Qt") then
@@ -243,7 +282,7 @@ function main(target, opt)
                     _add_includedirs(target, path.join(qt.includedir, private_dir, qt.sdkver))
                 end
             else
-                -- add defines
+                -- add definitions
                 target:add("defines", "QT_" .. framework:sub(3):upper() .. "_LIB")
 
                 -- add includedirs
@@ -282,9 +321,18 @@ function main(target, opt)
     end
     target:set("frameworks", local_frameworks)
 
-    -- add some static third-party links if exists, e.g. libqtmain.a, libqtfreetype.q, libqtlibpng.a
+    -- add some static third-party links if exists
     -- and exclude qt framework libraries, e.g. libQt5xxx.a, Qt5xxx.lib
-    target:add("syslinks", _find_static_links_3rd(target, qt.libdir, qt_sdkver, target:is_plat("windows") and "qt*.lib" or "libqt*.a"))
+    local libpattern
+    if qt_sdkver:ge("6.0") then
+        -- e.g. libQt6BundledFreetype.a on Qt6.x
+        -- @see https://github.com/xmake-io/xmake/issues/3572
+        libpattern = target:is_plat("windows") and "Qt*.lib" or "libQt*.a"
+    else
+        -- e.g. libqtmain.a, libqtfreetype.q, libqtlibpng.a on Qt5.x
+        libpattern = target:is_plat("windows") and "qt*.lib" or "libqt*.a"
+    end
+    target:add("syslinks", _find_static_links_3rd(target, qt.libdir, qt_sdkver, libpattern))
 
     -- add user syslinks
     if syslinks_user then
@@ -326,12 +374,20 @@ function main(target, opt)
         _add_includedirs(target, path.join(qt.mkspecsdir, "win32-msvc"))
         target:add("linkdirs", qt.libdir)
         target:add("syslinks", "ws2_32", "gdi32", "ole32", "advapi32", "shell32", "user32", "opengl32", "imm32", "winmm", "iphlpapi")
+        -- for debugger, https://github.com/xmake-io/xmake-vscode/issues/225
+        target:add("runenvs", "PATH", qt.bindir)
     elseif target:is_plat("mingw") then
         target:set("frameworks", nil)
-        -- we need fix it, because gcc maybe does not work on latest mingw when `-isystem D:\a\_temp\msys64\mingw64\include` is passed.
+        -- we need to fix it, because gcc maybe does not work on latest mingw when `-isystem D:\a\_temp\msys64\mingw64\include` is passed.
         -- and qt.includedir will be this path value when Qt sdk directory just is `D:\a\_temp\msys64\mingw64`
         -- @see https://github.com/msys2/MINGW-packages/issues/10761#issuecomment-1044302523
-        if qt.includedir and os.isdir(qt.includedir) then
+        if is_subhost("msys") then
+            local mingw_prefix = os.getenv("MINGW_PREFIX")
+            local mingw_includedir = path.normalize(path.join(mingw_prefix or "/", "include"))
+            if qt.includedir and qt.includedir and path.normalize(qt.includedir) ~= mingw_includedir then
+                _add_includedirs(target, qt.includedir)
+            end
+        else
             _add_includedirs(target, qt.includedir)
         end
         _add_includedirs(target, path.join(qt.mkspecsdir, "win32-g++"))
@@ -349,10 +405,36 @@ function main(target, opt)
         _add_includedirs(target, path.join(qt.mkspecsdir, "wasm-emscripten"))
         target:add("rpathdirs", qt.libdir)
         target:add("linkdirs", qt.libdir)
-        target:add("ldflags", "-s WASM=1", "-s FETCH=1", "-s FULL_ES2=1", "-s FULL_ES3=1", "-s USE_WEBGL2=1", "--bind")
-        target:add("ldflags", "-s ERROR_ON_UNDEFINED_SYMBOLS=1", "-s EXTRA_EXPORTED_RUNTIME_METHODS=[\"UTF16ToString\",\"stringToUTF16\"]", "-s ALLOW_MEMORY_GROWTH=1")
-        target:add("shflags", "-s WASM=1", "-s FETCH=1", "-s FULL_ES2=1", "-s FULL_ES3=1", "-s USE_WEBGL2=1", "--bind")
-        target:add("shflags", "-s ERROR_ON_UNDEFINED_SYMBOLS=1", "-s EXTRA_EXPORTED_RUNTIME_METHODS=[\"UTF16ToString\",\"stringToUTF16\"]", "-s ALLOW_MEMORY_GROWTH=1")
+        -- add prebuilt object files in qt sdk.
+        -- these files are located at lib/objects-Release/xxxmodule_resources_x/.rcc/xxxmodule.cpp.o
+        for _, framework in ipairs(qt_frameworks) do
+            local prefix = framework
+            if framework:startswith("Qt") then
+                prefix = framework:sub(3)
+            end
+            for _, filepath in ipairs(os.files(path.join(qt.libdir, "objects-*", prefix .. "_resources_*", ".rcc", "*.o"))) do
+                table.insert(target:objectfiles(), filepath)
+            end
+        end
+        target:add("ldflags", "-s FETCH=1", "-s ERROR_ON_UNDEFINED_SYMBOLS=1", "-s ALLOW_MEMORY_GROWTH=1", "--bind")
+        target:add("shflags", "-s FETCH=1", "-s ERROR_ON_UNDEFINED_SYMBOLS=1", "-s ALLOW_MEMORY_GROWTH=1", "--bind")
+        if qt_sdkver:ge("6.0") then
+            -- @see https://github.com/xmake-io/xmake/issues/4137
+            target:add("ldflags", "-s MAX_WEBGL_VERSION=2", "-s WASM_BIGINT=1", "-s DISABLE_EXCEPTION_CATCHING=1")
+            target:add("ldflags", "-sASYNCIFY_IMPORTS=qt_asyncify_suspend_js,qt_asyncify_resume_js")
+            target:add("ldflags", "-s EXPORTED_RUNTIME_METHODS=UTF16ToString,stringToUTF16,JSEvents,specialHTMLTargets")
+            target:add("ldflags", "-s MODULARIZE=1", "-s EXPORT_NAME=createQtAppInstance")
+            target:add("shflags", "-s MAX_WEBGL_VERSION=2", "-s WASM_BIGINT=1", "-s DISABLE_EXCEPTION_CATCHING=1")
+            target:add("shflags", "-sASYNCIFY_IMPORTS=qt_asyncify_suspend_js,qt_asyncify_resume_js")
+            target:add("shflags", "-s EXPORTED_RUNTIME_METHODS=UTF16ToString,stringToUTF16,JSEvents,specialHTMLTargets")
+            target:add("shflags", "-s MODULARIZE=1", "-s EXPORT_NAME=createQtAppInstance")
+            target:set("extension", ".js")
+        else
+            target:add("ldflags", "-s WASM=1", "-s FULL_ES2=1", "-s FULL_ES3=1", "-s USE_WEBGL2=1")
+            target:add("ldflags", "-s EXPORTED_RUNTIME_METHODS=[\"UTF16ToString\",\"stringToUTF16\"]")
+            target:add("shflags", "-s WASM=1", "-s FULL_ES2=1", "-s FULL_ES3=1", "-s USE_WEBGL2=1")
+            target:add("shflags", "-s EXPORTED_RUNTIME_METHODS=[\"UTF16ToString\",\"stringToUTF16\"]")
+        end
     end
 
     -- is gui application?
@@ -376,5 +458,11 @@ function main(target, opt)
             target:add("ldflags", "-mwindows", {force = true})
         end
     end
+
+    -- set default runtime
+    -- @see https://github.com/xmake-io/xmake/issues/4161
+    if not target:get("runtimes") then
+        target:set("runtimes", is_mode("debug") and "MDd" or "MD")
+    end
 end
 
