@@ -61,10 +61,10 @@ rule("qt.moc")
         -- get values from target
         -- @see https://github.com/xmake-io/xmake/issues/3930
         local function _get_values_from_target(target, name)
-            local values = table.wrap(target:get(name))
-            table.join2(values, target:get_from_opts(name))
-            table.join2(values, target:get_from_pkgs(name))
-            table.join2(values, target:get_from_deps(name, {interface = true}))
+            local values = {}
+            for _, value in ipairs((target:get_from(name, "*"))) do
+                table.join2(values, value)
+            end
             return table.unique(values)
         end
 
